/*
** Copyright (c) 2002 D. Richard Hipp
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
** 
** You should have received a copy of the GNU General Public
** License along with this library; if not, write to the
** Free Software Foundation, Inc., 59 Temple Place - Suite 330,
** Boston, MA  02111-1307, USA.
**
** Author contact information:
**   drh@hwaci.com
**   http://www.hwaci.com/drh/
**
*******************************************************************************
**
** This file contains code used to generate the "timeline"
** family of reports
*/
#include <time.h>
#include "config.h"
#include "timeline.h"

/*
** Parse a text time description
*/
static time_t parse_time(const char *zTime){
  char zMonth[30];
  struct tm sTm;
  int y, d, h, m, s;
  int i, n;
  static struct {
    char *zName;
    int iMon;
  } aMName[] = {
    { "january",    1 },
    { "february",   2 },
    { "march",      3 },
    { "april",      4 },
    { "may",        5 },
    { "june",       6 },
    { "july",       7 },
    { "jly",        7 },
    { "august",     8 },
    { "sepember",   9 },
    { "october",   10 },
    { "november",  11 },
    { "december",  12 },
  };

  y = d = h = m = s = 0;
  if( sscanf(zTime, "%d-%20[^-]-%d %d:%d:%d",
         &y, zMonth, &d, &h, &m, &s)<3 ) return 0;
  for(i=0; zMonth[i]; i++){
    if( isupper(zMonth[i]) ) zMonth[i] = tolower(zMonth[i]);
  }
  n = strlen(zMonth);
  memset(&sTm, 0, sizeof(sTm));
  for(i=0; i<sizeof(aMName)/sizeof(aMName[0]); i++){
    if( strncmp(zMonth, aMName[i].zName, n)==0 ){
      sTm.tm_mon = aMName[i].iMon - 1;
      break;
    }
  }
#if CVSTRAC_I18N
  if( i>=sizeof(aMName)/sizeof(aMName[0]) ){
    for(i=0; i<12; i++){
      if( strncasecmp(zMonth, nl_langinfo(ABMON_1+i), n)==0
               || strncasecmp(zMonth, nl_langinfo(MON_1+i),n )==0 ){
        sTm.tm_mon = i;
        break;
      }
    }
  }
#endif
  sTm.tm_year = y - 1900;
  sTm.tm_mday = d;
  sTm.tm_hour = h;
  sTm.tm_min = m;
  sTm.tm_sec = s;
  sTm.tm_isdst = -1;
  return mktime(&sTm);
}

/*
** WEBPAGE: /timeline
*/
void timeline_page(void){
  static const char *azLink[] = { 
     "index", "Home",
     "search?t=1&c=1", "Search",
     "tktnew", "New Ticket",
     0
  };
  const char *zEnd;       /* Day at which timeline ends */
  time_t begin, end;      /* Beginning and ending times for the timeline */
  char **az;  
  time_t thisDate;
  int thisDay, lastDay;
  int inTable = 0;
  int i;
  struct tm *pTm;
  int len = 0;
  int days = 30;
  int showM;       /* Show milestones if 1.  Do not show if 0 */
  int showC;       /* Show all checkins if 2, branch checkis if 1, none if 0 */
  int showS;       /* 0: no status changes 1: active & new  2: everything */
  int showA;       /* Show assignment changes if 1 */
  int showW;       /* Show changes to Wiki pages if 1 */
  int divM;        /* Divide timeline by milestones if 1 */
  int divT;        /* Divide timeline by days if 1 */
  const char *zCkinPrfx;   /* Only show checkins of files with this prefix */
  char zDate[200];
  char zSQL[3000];

  login_check_credentials();
  if( !g.okRead && !g.okCheckout ){ cgi_redirect("index"); return; }
  if( !g.okNewTkt ) azLink[2] = 0;
  history_update();
  if( P("x") ){
    showM = PD("m","0")[0] - '0';
    showC = PD("c","0")[0] - '0';
    showS = PD("s","0")[0] - '0';
    showA = PD("a","0")[0] - '0';
    showW = PD("w","0")[0] - '0';
    divM = PD("dm","0")[0] - '0';
    divT = PD("dt","0")[0] - '0';
    if( P("set") && P("d") && g.okWrite ){
      char zVal[12];
      zVal[0] = showA + '0';
      zVal[1] = showC + '0';
      zVal[2] = showM + '0';
      zVal[3] = showS + '0';
      zVal[4] = showW + '0';
      zVal[5] = divM + divT*2 + '0';
      zVal[6] = 0;
      db_execute("REPLACE INTO config(name,value) "
                 "VALUES('timeline_format','%s%s')", zVal, P("d"));
    }
  }else{
    char *zFormat;
    zFormat = db_short_query("SELECT value FROM config "
                             " WHERE name='timeline_format'");
    if( zFormat && zFormat[0] ){
      showA = zFormat[0]-'0';
      showC = zFormat[1]-'0';
      showM = zFormat[2]-'0';
      showS = zFormat[3]-'0';
      showW = zFormat[4]-'0';
      divM = zFormat[5]-'0';
      divT = divM/2;
      divM %= 2;
      days = atoi(&zFormat[6]);
      free(zFormat);
    }else{
      showM = showS = 1;
      showC = 2;
      showA = showW = 0;
      divM = 0;
      divT = 1;
    }
  }
  zCkinPrfx = P("px");
  if( zCkinPrfx==0 || zCkinPrfx[0]==0 || showC==0 ){
    zCkinPrfx = 0;
  }else{
    zCkinPrfx = sqlite_mprintf("%q", zCkinPrfx);
  }
  if( !g.okRead ){
    showS = showA = 0;
  }
  zEnd = P("e");
  if( zEnd==0 || strcmp(zEnd,"today")==0 || (end = parse_time(zEnd))==0 ){
    time(&end);
  }
  pTm = localtime(&end);
  pTm->tm_hour = 0;
  pTm->tm_min = 0;
  pTm->tm_sec = 0;
  end = mktime(pTm);
  i = atoi(PD("d","0"));
  if( i>0 ) days = i;
  begin = end - 3600*24*days;
  end += 3600*24 - 1;
  if( showM || (showC && zCkinPrfx==0) || divM ){
    sprintf(zSQL,
       "SELECT "
       "  date AS 'time', 1 AS 'type', user, milestone, branch, cn, message "
       "FROM chng WHERE date<=%d AND date>=%d",
       (int)end, (int)begin);
    len = strlen(zSQL);
    if( showM==0 && divM==0 ){
      sprintf(&zSQL[len]," AND NOT milestone");
      len += strlen(&zSQL[len]);
    }
    if( showC==0 || zCkinPrfx  ){
      sprintf(&zSQL[len]," AND milestone");
      len += strlen(&zSQL[len]);
    }else if( showC==1 ){
      sprintf(&zSQL[len]," AND (milestone OR branch!='')");
      len += strlen(&zSQL[len]);
    }
  }
  if( zCkinPrfx ){
    if( len>0 ){
      sprintf(&zSQL[len]," UNION ALL ");
      len += strlen(&zSQL[len]);
    }
    sprintf(&zSQL[len],
       "SELECT DISTINCT "
       "  chng.date AS 'time',"
       "  1 AS type,"
       "  chng.user,"
       "  0 AS milestone,"
       "  chng.branch,"
       "  chng.cn,"
       "  chng.message "
       "FROM chng, filechng "
       "WHERE filechng.cn=chng.cn AND filechng.filename LIKE '%s%%'"
       "  AND chng.date<=%d AND chng.date>=%d ",
       zCkinPrfx, (int)end, (int)begin);
    len += strlen(&zSQL[len]);
  }
  if( showS || showA ){
    if( len>0 ){
      sprintf(&zSQL[len]," UNION ALL ");
      len += strlen(&zSQL[len]);
    }
    sprintf(&zSQL[len],
       "SELECT origtime AS 'time', 2 AS 'type', owner, type, NULL, tn, title "
       "FROM ticket WHERE origtime<=%d AND origtime>=%d",
       (int)end, (int)begin);
    len += strlen(&zSQL[len]);
    if( showS ){
      sprintf(&zSQL[len],
         " UNION ALL "
         "SELECT chngtime AS 'time', 3 AS 'type', user, oldval AS 'aux1', "
         "       newval AS 'aux2', tn AS 'ref', NULL as 'text' "
         "FROM tktchng "
         "WHERE fieldid='status' AND chngtime<=%d AND chngtime>=%d ",
         (int)end, (int)begin);
      len += strlen(&zSQL[len]);
      if( showS==1 ){
        sprintf(&zSQL[len],
          " AND ("
          "(newval IN ('new','active') AND oldval NOT IN ('new','active')) OR"
          "(newval NOT IN ('new','active') AND oldval IN ('new','active')))");
        len += strlen(&zSQL[len]);
      }
    }
    if( showS==3 ){
      sprintf(&zSQL[len],
         " UNION ALL "
         "SELECT chngtime AS 'time', 6 AS 'type', user, '' AS 'aux1', "
         "       '' AS 'aux2', tn AS 'ref', NULL as 'text' "
         "FROM tktchng "
         "WHERE fieldid!='status' AND fieldid!='assignedto'"
         "  AND chngtime<=%d AND chngtime>=%d ",
         (int)end, (int)begin);
      len += strlen(&zSQL[len]);
    }
    if( showA ){
      sprintf(&zSQL[len],
         " UNION ALL "
         "SELECT chngtime AS 'time', 4 AS 'type', user, oldval AS 'aux1', "
         "       newval AS 'aux2', tn AS 'ref', NULL as 'text' "
         "FROM tktchng "
         "WHERE fieldid='assignedto' AND chngtime<=%d AND chngtime>=%d",
         (int)end, (int)begin);
      len += strlen(&zSQL[len]);
    }
  }
  if( showW ){
    if( len>0 ){
      sprintf(&zSQL[len]," UNION ALL ");
      len += strlen(&zSQL[len]);
    }
    sprintf(&zSQL[len],
       "SELECT -invtime AS 'time', 5 AS 'type', who, NULL, NULL, "
       "       name as 'ref', NULL as 'text' "
       "FROM wiki "
       "WHERE invtime>=%d AND invtime<=%d",
       -(int)end, -(int)begin);
    len += strlen(&zSQL[len]);
  }
  if( len==0 ){
    static char *azDummy[] = { 0 };
    az = azDummy;
  }else{
    sprintf(&zSQL[len], " ORDER BY time DESC, type");
    az = db_query("%s",zSQL);
  }
  lastDay = 0;
  common_header("Timeline", azLink);
  if( P("debug1") ){
    @ <p>%h(zSQL)</p><hr>
  }
  for(i=0; az[i]; i+=7){
    char *zIcon = 0;
    int iconWidth = 0;
    char *zBg = "";
    char *zFg = 0;
    char *zMsg;
    char zPrefix[400];
    char zSuffix[200];
    char zMsgBuf[5];

    zMsgBuf[0] = ' ';
    zMsgBuf[1] = 0;
    zMsg = zMsgBuf;
    thisDate = atoi(az[i]);
    pTm = localtime(&thisDate);
    thisDay = (pTm->tm_year+1900)*1000 + pTm->tm_yday;
    if( !inTable ){
      @ <table cellspacing=0 border=0 cellpadding=0>
      inTable = 1;
    }
    if( thisDay!=lastDay && divT ){
      strftime(zDate, sizeof(zDate), "%A, %Y-%b-%d", pTm);
      @ <tr><td colspan=3>
      @ <table cellpadding=2 border=0>
      @ <tr><td bgcolor="%s(BORDER1)" class="border1">
      @ <table cellpadding=2 cellspacing=0 border=0><tr>
      @   <td bgcolor="%s(BG1)" class="bkgnd1">%s(zDate)</td>
      @ </tr></table>
      @ </td></tr></table>
      @ </td></tr>
      lastDay = thisDay;
    }
    if( divM && az[i+1][0]=='1' && az[i+3][0] && az[i+3][0]=='1' ){
      @ <tr><td colspan=3>
      @ <table cellpadding=2 border=0>
      @ <tr><td bgcolor="%s(BORDER2)" class="border2">
      @ <table cellpadding=2 cellspacing=0 border=0><tr>
      @   <td bgcolor="%s(BG2)" class="bkgnd2">%h(az[i+6])</td>
      @ </tr></table>
      @ </td></tr></table>
      @ </td></tr>
      lastDay = thisDay-1;
    }
    if( az[i+1][0]=='1' && !showM && az[i+3][0] && az[i+3][0]!='0' ) continue;
    if( divT || thisDay==lastDay ){
      strftime(zDate, sizeof(zDate), "%H:%M", pTm);
    }else{
      strftime(zDate, sizeof(zDate), "%Y-%b-%d %H:%M", pTm);
      lastDay = thisDay;
    }
    @ <tr>
    @ <td valign="top" align="right" width=%d(160-divT*100)>%s(zDate)</td>
    zPrefix[0] = 0;
    zSuffix[0] = 0;
    switch( atoi(az[i+1]) ){
      case 1: { /* A check-in or milestone */
        zMsg = az[i+6];
        iconWidth = 9;
        if( az[i+3][0] && az[i+3][0]!='0' ){
          zIcon = "dia.gif";
          sprintf(zPrefix, "Milestone [%.20s]: ", az[i+5]);
          /* zBg = " bgcolor=\"#e3ffe3\""; */
          /* zFg = "#009100"; */
        }else{
          zIcon = "dot.gif";
          if( az[i+4][0] ){
            sprintf(zPrefix, "Check-in [%.20s] on branch %.50s: ",
                    az[i+5], az[i+4]);
            /* zBg = " bgcolor=\"#dddddd\""; */
            /* zFg = "#909090"; */
          }else{
            sprintf(zPrefix, "Check-in [%.20s]: ", az[i+5]);
          }
          sprintf(zSuffix, " (By %.30s)", az[i+2]);
        }
        break;
      }
      case 2: {  /* A new ticket was created */
        zMsg = az[i+6];
        sprintf(zPrefix, "Create ticket #%.20s: ", az[i+5]);
        sprintf(zSuffix, " (By %.30s)", az[i+2]);
        zIcon = "x.gif";
        iconWidth = 9;
        break;
      }
      case 3: {  /* The status field of a ticket changed */
        char zType[50];
        sprintf(zType,"%.30s",az[i+4]);
        if( islower(zType[0]) ) zType[0] = toupper(zType[0]);
        sprintf(zPrefix, "%.30s ticket #%.20s, was %.20s.",
             zType, az[i+5], az[i+3]);
        sprintf(zSuffix, "(By %.30s)", az[i+2]);
        if( strcmp(az[i+4],"active")==0 || strcmp(az[i+4],"new")==0 ){
          zIcon = "x.gif";
        }else if( (strcmp(az[i+3],"active")==0 || strcmp(az[i+3],"new")==0) 
           && (strcmp(az[i+4],"fixed")==0 || strcmp(az[i+4],"closed")==0) ){
          zIcon = "ck.gif";
        }else{
          zIcon = "dia.gif";
        }
        iconWidth = 9;
        if( az[i+7] && atoi(az[i+8])==4 && strcmp(az[i],az[i+7])==0
            && strcmp(az[i+5],az[i+12])==0 ){
          char *zTail = &zPrefix[strlen(zPrefix)];
          i += 7;
          if( az[i+4][0]==0 ){
            sprintf(zTail, " Unassign from %.50s.", az[i+3]);
          }else if( az[i+3][0]==0 ){
            sprintf(zTail, " Assign to %.50s.", az[i+4]);
          }else{
            sprintf(zTail, " Reassign from %.50s to %.50s", az[i+3], az[i+4]); 
          }
        }
        if( az[i+7] && atoi(az[i+8])==6 && strcmp(az[i],az[i+7])==0
            && strcmp(az[i+5],az[i+12])==0 ){
          char *zTail = &zPrefix[strlen(zPrefix)];
          i += 7;
          sprintf(zTail, " Plus other changes.");
          while( az[i+7] && atoi(az[i+8])==6 && strcmp(az[i],az[i+7])==0
                 && strcmp(az[i+5],az[i+12])==0 ){
            i += 7;
          }
        }
        break;
      }
      case 4: {  /* The assigned-to field of a ticket changed */
        if( az[i+4][0]==0 ){
          sprintf(zPrefix, "Unassign ticket #%.20s from %.50s.",
             az[i+5], az[i+3]);
        }else if( az[i+3][0]==0 ){
          sprintf(zPrefix, "Assign ticket #%.20s to %.50s.",
             az[i+5], az[i+4]);
        }else{
          sprintf(zPrefix, "Reassign ticket #%.20s from %.50s to %.50s",
             az[i+5], az[i+3], az[i+4]); 
        }
        sprintf(zSuffix, "(By %.30s)", az[i+2]);
        zIcon = "dia.gif";
        iconWidth = 9;
        break;
      }
      case 5: {   /* Changes to a Wiki page */
        sprintf(zPrefix, "Wiki page %s ", az[i+5]);
        sprintf(zSuffix, "edited by %.30s", az[i+2]);
        zIcon = "dia.gif";
        iconWidth = 9;
        break;
      }
      case 6: {  /* Changes to a ticket other than status or assignment */
        sprintf(zPrefix, "Changes to ticket #%.20s", az[i+5]);
        sprintf(zSuffix, "(By %.30s)", az[i+2]);
        zIcon = "dia.gif";
        iconWidth = 9;
        while( az[i+7] && atoi(az[i+8])==6 && strcmp(az[i],az[i+7])==0
               && strcmp(az[i+5],az[i+12])==0 ){
          i += 7;
        }
        break;
      }
      default:
        /* Cannot happen */
        break;
    }
    @ <td valign="top" align="center" width=30>
    if( zIcon ){
      int w = iconWidth;
      @   <img src="%s(zIcon)" width=%d(w) height=9 align="bottom" alt="*">
    }
    @ </td>
    @ <td valign="top"%s(zBg)>
    if( zFg ){
      @ <font color="%s(zFg)">
    }
    output_formatted(zPrefix);
    if( output_trim_message(zMsg, MN_CKIN_MSG, MX_CKIN_MSG) ){
      output_formatted(zMsg);
      @ &nbsp;[...]
    }else{
      output_formatted(zMsg);
    }
    output_formatted(zSuffix);
    if( zFg ){
      @ </font>
    }
    @ </td></tr>
  }
  if( inTable ){
    @ </table>
    inTable = 0;
  }
  @ <hr>
  @ <form method="GET" action="timeline">
  @ <table>
  @ <tr><td colspan=3>
  @ Show a timeline of
  @ <input type="text" name="d" value="%d(days)" size=3> days
  @ going backwards from
  pTm = localtime(&end);
  strftime(zDate, sizeof(zDate), "%Y-%b-%d", pTm);
  @ <input type="text" name="e" value="%s(zDate)" size=11>
  @ </td></tr>
  @ <tr>
  if( g.okCheckout ){
    zCkinPrfx = P("px");
    @ <td align="left" valign="top">
    @ <input type="radio" name="c"%s(showC==2?" checked":"") value="2">
    @   Show all check-ins<br>
    @ <input type="radio" name="c"%s(showC==1?" checked":"") value="1">
    @   Show only branch check-ins<br>
    @ <input type="radio" name="c"%s(showC==0?" checked":"") value="0">
    @   Show no check-ins<br>
    @ Only show check-ins of files with this prefix:<br>
    @ <input type="text" name="px" value="%h(zCkinPrfx?zCkinPrfx:"")" size="40">
    @ </td><td width=50>&nbsp;</td>
  }
  if( g.okRead ){
    @ <td valign="top">
    @ <input type="radio" name="s"%s(showS==3?" checked":"") value="3">
    @   Show all ticket changes of any kind<br>
    @ <input type="radio" name="s"%s(showS==2?" checked":"") value="2">
    @   Show all ticket status changes<br>
    @ <input type="radio" name="s"%s(showS==1?" checked":"") value="1">
    @   Show only "active" and "new" status changes<br>
    @ <input type="radio" name="s"%s(showS==0?" checked":"") value="0">
    @   Show no ticket status changes<br>
    @ </td></tr>
  }
  @ <tr><td valign="top">
  @ <input type="checkbox" name="dm"%s(divM?" checked":"") value="1">
  @   Divide timeline by milestones<br>
  @ <input type="checkbox" name="dt"%s(divT?" checked":"") value="1">
  @   Divide timeline by days<br>
  @ <input type="checkbox" name="debug1"%s(P("debug1")?" checked":"")
  @  value="1"> Show the SQL used to generate the timeline.
  @ </td><td width=50>&nbsp;</td><td>
  if( g.okRead ){
    @ <input type="checkbox" name="a"%s(showA?" checked":"") value="1">
    @   Show assignment changes<br>
  }
  if( g.okCheckout ){
    @ <input type="checkbox" name="m"%s(showM?" checked":"") value="1">
    @   Show milestones<br>
  }
  @ <input type="checkbox" name="w"%s(showW?" checked":"") value="1">
  @   Show Wiki edits
  @ </td></tr>
  @ <tr><td colspan=3>
  @ <input type="hidden" name="x" value="1">
  @ <input type="submit" value="Show Timeline">
  if( g.okAdmin ){
    @ &nbsp;&nbsp;&nbsp;
    @ <input type="submit" name="set" value="Make Default">
  }
  @ </td></tr>
  @ </table>
  @ </form>
  common_footer();
}

/*
** Given a file version number, compute the previous version number.
** The new version number overwrites the old one.
**
** Examples:  "1.12" becomes "1.11".  "1.22.2.1" becomes "1.22".
**
** The special case of "1.1" becomes "".
*/
void previous_version(char *zVers){
  int j, x;
  int n = strlen(zVers);
  for(j=n-2; j>=0 && zVers[j]!='.'; j--){}
  j++;
  x = atoi(&zVers[j]);
  if( x>1 ){
    sprintf(&zVers[j],"%d",x-1);
  }else{
    for(j=j-2; j>0 && zVers[j]!='.'; j--){}
    zVers[j] = 0;
  }
}

/*
** Generate a page that shows specifics of a particular checkin.
**
** WEBPAGE: /chngview
*/
void checkin_view(void){
  char **az, **azFile, **azTkt;
  int cn;
  time_t tx;
  struct tm *pTm;
  int i;
  int cnt;
  char zDate[200];
  char zDateUTC[200];
  static const char *azLink[] = { 
     "index", "Home",
     "timeline", "Timeline",
     "search?c=1", "Search",
     0
  };

  login_check_credentials();
  if( !g.okRead ){ cgi_redirect("index"); return; }
  cn = atoi(PD("cn","0"));
  az = db_query("SELECT date, branch, milestone, user, message "
                "FROM chng WHERE cn=%d", cn);
  if( az[0]==0 ){ cgi_redirect("index"); return; }
  azFile = db_query("SELECT filename, vers, nins, ndel "
                    "FROM filechng WHERE cn=%d ORDER BY filename", cn);
  azTkt = db_query(
     "SELECT ticket.tn, ticket.title FROM xref, ticket "
     "WHERE xref.cn=%d AND xref.tn=ticket.tn "
     "ORDER BY ticket.tn", cn);
  common_header("Check-in [%d]", azLink, cn);
  tx = (time_t)atoi(az[0]);
  pTm = localtime(&tx);
  strftime(zDate, sizeof(zDate), "%Y-%b-%d %H:%M:%S", pTm);
  pTm = gmtime(&tx);
  strftime(zDateUTC, sizeof(zDateUTC), "%Y-%b-%d %H:%M:%S", pTm);
  @ <table cellpadding=1 cellspacing=0 width="100%%">
  @ <tr><td align="right" valign="top">Check-in Number:</td><td>%d(cn)</td>
  @   <td align="right" rowspan=4 valign="top"><a href="taghints?cn=%d(cn)">
  @        Tagging and branching hints</a></td></tr>
  @ <tr><td align="right" valign="top">Date:</td>
  @   <td>%s(zDate) (local)
  @   <br>%s(zDateUTC) (UTC)</td></tr>
  @ <tr><td align="right">User:</td><td>%h(az[3])</td></tr>
  @ <tr><td align="right">Branch:</td><td>%h(az[1])</td></tr>
  @ <tr><td align="right" valign="top">Comment:</td><td colspan=2>
  output_formatted(az[4]);
  if( g.okWrite && g.okCheckin ){
    @
    @ <a href="chngedit?cn=%d(cn)">(edit)</a>
  }
  @ </td></tr>
  @ <tr><td align="right" valign="top">Tickets:</td><td colspan=2>
  if( azTkt[0]!=0 ){
    @ <table cellpadding=0 cellspacing=0 border=0>
    for(i=0; azTkt[i]; i+=2 ){
      @ <tr><td align="right" valign="top">
      @ <a href="tktview?tn=%h(azTkt[i])">#%h(azTkt[i])</a></td>
      @ <td width=8></td>
      @ <td>%h(azTkt[i+1])</td></tr>
    }
    @ </table></td></tr>
  }
  @ <tr><td align="right" valign="top">Files:</td><td colspan=2>
  @ <table cellpadding=0 cellspacing=0 border=0>
  cnt = 0;
  for(i=0; azFile[i]; i+=4){
    char zPVers[100];
    sprintf(zPVers, "%.90s", azFile[i+1]);
    previous_version(zPVers);
    if( g.okCheckout ){
      @ <tr><td><a href="rlog?f=%T(azFile[i])">%h(azFile[i])</a>
    }else{
      @ <tr><td>%h(azFile[i])
    }
    @ &nbsp;&nbsp;&nbsp;&nbsp;</td>
    if( strcmp(azFile[i+1],"1.1")==0 ){
      cnt++;
      @ <td>New File</td>
    }else{
      cnt++;
      @ <td>%h(zPVers)->%h(azFile[i+1])</td>
      if( azFile[i+2][0] && azFile[i+3][0] ){
        @ <td>&nbsp;&nbsp;&nbsp
        @ %h(azFile[i+2]) inserted, %h(azFile[i+3]) deleted </td>
      }
    }
    @ </tr>
  }
  @ </table>
  @ </td></tr></table>
  if( cnt>0 && g.okCheckout ){
    char *zRoot = db_short_query("SELECT value FROM config "
                                 "WHERE name='cvsroot'");
    if( zRoot==0 || access(zRoot,0) ) azFile[0] = 0;
    for(i=0; azFile[i]; i+=4){
      FILE *in;
      char *zCmd;
      char *zFile;
      char zPVers[100];
      char zLine[1000];
      sprintf(zPVers, "%.90s", azFile[i+1]);
      previous_version(zPVers);
      zFile = find_repository_file(zRoot, azFile[i]);
      if( zFile==0 ) continue;
      if( zPVers[0]==0 ){
        zCmd = mprintf("co -q -p%s \"%s\" | diff -c /dev/null -",
            azFile[i+1], zFile);
      }else{
        zCmd = mprintf("rcsdiff -q -r%s -r%s -u \"%s\"",
            zPVers, azFile[i+1], zFile);
      }
      in = popen(zCmd, "r");
      free(zCmd);
      free(zFile);
      if( in==0 ) continue;
      @ <hr>
      @ %h(azFile[i]) &nbsp;&nbsp;&nbsp; %h(zPVers) -> %h(azFile[i+1])<br>
      @ <pre>
      while( fgets(zLine, sizeof(zLine), in) ){
        cgi_printf("%h", zLine);
      }
      @ </pre>
      pclose(in);
    }
  }
  common_footer();
}

/*
** WEBPAGE: /chngedit
*/
void change_edit(void){
  static const char *azLink[] = {
     "index", "Home",
     0, "Cancel",
     0
  };
  char zCancel[200];
  int cn;
  const char *zMsg;
  char **az;

  login_check_credentials();
  if( !g.okWrite || !g.okCheckin ){
    cgi_redirect("index");
    return;
  }
  cn = atoi(PD("cn","0"));
  if( cn<=0 ){
    cgi_redirect("index");
    return;
  }
  sprintf(zCancel,"chngview?cn=%d", cn);
  if( P("can") ){
    cgi_redirect(zCancel);
    return;
  }
  zMsg = P("m");
  if( zMsg ){
    db_execute("UPDATE chng SET message='%q' WHERE cn=%d", zMsg, cn);
    cgi_redirect(zCancel);
    return;
  }
  az = db_query("SELECT message, milestone FROM chng WHERE cn=%d", cn);
  zMsg = az[0];
  if( zMsg==0 ) zMsg = "";
  if( az[1][0] && atoi(az[1]) ){
    milestone_edit();
    return;
  }
  azLink[2] = zCancel;
  common_header("Edit Check-in [%d]", azLink, cn);
  @ <form action="chngedit" method="POST">
  @ <input type="hidden" name="cn" value="%d(cn)">
  @ Edit the change message and press "OK":<br>
  @ <textarea name="m" rows="8" cols="80" wrap="virtual">
  @ %h(zMsg)
  @ </textarea>
  @ <blockquote>
  @ <input type="submit" name="ok" value="OK">
  @ <input type="submit" name="can" value="Cancel">
  @ </blockquote>
  @ </form>
  common_footer();
}

/*
** Generate a page that shows how to create a new tag or branch
**
** WEBPAGE: /taghints
*/
void tag_hints(void){
  char **az;
  int cn;
  time_t tx;
  struct tm *pTm;
  char zDateUTC[200];
  char zLink[200];
  static const char *azLink[] = { 
     "index", "Home",
     "timeline", "Timeline",
     0, "Back",
     0
  };

  login_check_credentials();
  if( !g.okRead ){ cgi_redirect("index"); return; }
  cn = atoi(PD("cn","0"));
  az = db_query("SELECT date, branch, milestone, user, message "
                "FROM chng WHERE cn=%d", cn);
  if( az[0]==0 ){ cgi_redirect("index"); return; }
  sprintf(zLink,"chngview?cn=%d",cn);
  azLink[4] = zLink;
  common_header("Tagging And Branching Hints", azLink);
  tx = (time_t)atoi(az[0])-1;
  pTm = gmtime(&tx);
  strftime(zDateUTC, sizeof(zDateUTC), "%Y-%m-%d %H:%M:%S UTC", pTm);
  @ <p>To create a tag that occurs <i>before</i> 
  @ check-in <a href="%s(zLink)">[%d(cn)]</a>, go to the root of
  @ the source tree and enter the following CVS command:</p>
  @ <blockquote>
  @ <tt>cvs rtag -D '%s(zDateUTC)' </tt><i>&lt;tag-name&gt;</i><tt> .</tt>
  @ </blockquote>
  @
  @ <p>Be careful to include the dot (".") at the end of the line!
  @ To create a branch that occurs before the checking, enter this 
  @ command:</p>
  @ <blockquote>
  @ <tt>cvs rtag -b -D '%s(zDateUTC)' </tt><i>&lt;branch-name&gt;</i><tt> .</tt>
  @ </blockquote>
  common_footer();
}

/*
** A webpage to create a new milestone
**
** WEBPAGE: /msnew
** WEBPAGE: /msedit
*/
void milestone_edit(void){
  static const char *azLink[] = {
     "index", "Home",
     "timeline", "Timeline",
     0
  };
  const char *zTime;
  const char *zMsg;
  time_t tm;
  int cn;
  int mtype;
  const char *zMType;
  char **az;
  struct tm *pTm;
  char zDate[200];

  login_check_credentials();
  if( !g.okWrite || !g.okCheckin ){ cgi_redirect("index"); return; }
  zTime = P("t");
  if( zTime==0 || (tm = parse_time(zTime))==0 ){
    zTime = "";
  }
  cn = atoi(PD("cn","0"));
  zMsg = remove_blank_lines(PD("m",""));
  zMType = PD("y","0");
  mtype = atoi(zMType);
  if( tm>0 && zTime[0] && zMsg[0] && mtype>0 ){
    if( P("del") && cn>0 ){
      db_execute("DELETE FROM chng WHERE cn=%d AND milestone", cn);
    }else if( cn>0 ){
      db_execute("UPDATE chng SET "
                 "date=%d, user='%q', message='%q', milestone=%d "
                 "WHERE cn=%d",
         tm, g.zUser, zMsg, mtype, cn);
    }else{
      db_execute("INSERT INTO chng(date,milestone,user,message) "
         "VALUES(%d,%d,'%q','%q')",
         tm, mtype, g.zUser, zMsg);
    }
    cgi_redirect("index");
    return;
  }
  az = cn<=0 ? 0 :
    db_query("SELECT date, milestone, message FROM chng "
             "WHERE cn=%d AND milestone", cn);
  if( az && az[0] ){
    tm = atoi(az[0]);
    zMType = az[1];
    mtype = atoi(zMType);
    zMsg = az[2];
  }else{
    time(&tm);
    zMsg = "";
    cn = 0;
    zMType = "1";
    mtype = 1;
  }
  if( cn>0 ){
    common_header("Edit Milestone", azLink);
  }else{
    common_header("Create New Milestone", azLink);
  }
  @ <form action="msedit" method="POST">
  if( cn>0 ){
    @ <input type="hidden" name="cn" value="%d(cn)">
  }
  pTm = localtime(&tm);
  strftime(zDate, sizeof(zDate), "%Y-%b-%d %H:%M:%S", pTm);
  @ <table>
  @ <tr><td align="right">Date &amp; Time:</td>
  @    <td><input type="text" name="t" value="%s(zDate)" size=20></td>
  @ <td width=20>&nbsp;</td>
  @ <td>Type: 
  cgi_optionmenu(2,"y", zMType, "Release", "1", "Event", "2", 0);
  @ </td></tr>
  @ <tr><td align="right">Comment:</td>
  @    <td colspan=3>
  @    <input type="text" name="m" value="%h(zMsg)" size=70>
  @ </td></tr>
  @ <tr><td colspan=4 align="center">
  @   <input type="submit" value="Submit">
  if( cn>0 ){
    @   &nbsp;&nbsp;&nbsp;
    @   <input type="submit" value="Delete" name="del">
  }
  @ </td></tr></table>
  @ </form>
  common_footer();
}
